import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * A PlatformJumper is a PlatformSolid that can (theoretically) move and jump on its own.
 * How it does this and under what circumstances will tend to vary, but in the most general
 * sense, the PlatformJumper is the stock physics object for this scenario.
 * 
 * @author BenMcBen
 * @version 10/26/2014
 */
public class PlatformJumper extends PlatformSolid
{
    protected int xVel;
    protected int yVel;
    protected int grav;
    protected int fric;
    protected int trmX;
    protected int trmY;
    protected int will;
    protected int jstr;
    protected int step;
    protected int ltcy;
    protected boolean horMirrored;
    public String represent() {
        return "PlJu";
    }
    public PlatformJumper() {
        xVel = 0;
        yVel = 0;
        grav = 1;
        fric = 1;
        trmX = 12;
        trmY = 16;
        will = 2;
        jstr = 16;
        step = 6;
        ltcy = 64;
        horMirrored = false;
    }
    public boolean pressingRight() {
        return Greenfoot.isKeyDown("right") || Greenfoot.isKeyDown("d");
    }
    public boolean pressingLeft() {
        return Greenfoot.isKeyDown("left") || Greenfoot.isKeyDown("a");
    }
    public boolean pressingJump() {
        return Greenfoot.isKeyDown("up") || Greenfoot.isKeyDown("w");
    }
    public int getXVel() {
        return xVel;
    }
    public int getYVel() {
        return yVel;
    }
    public void setVel(int newXVel, int newYVel) {
        xVel = newXVel;
        yVel = newYVel;
    }
    public void goRight() {
        if (xVel < trmX) {
            xVel += will;
        }
        if (horMirrored) {
            getImage().mirrorHorizontally();
            horMirrored = false;
        }
    }
    public void goLeft() {
        if (xVel > -trmX) {
            xVel -= will;
        }
        if (!horMirrored) {
            getImage().mirrorHorizontally();
            horMirrored = true;
        }
    }
    public void tryJump() {
        int origX = getX();
        int origY = getY();
        setLocation(origX, origY + trmY);
        if (yVel == 0 && isTouching(PlatformSolid.class)) {
            yVel = -jstr;
        }
        setLocation(origX, origY);
    }
    public void applyGravity() {
        if (yVel < trmY) {
            if (yVel + grav == 0) {
                yVel += grav;
            }
            yVel += grav;
        }
    }
    public void applyFriction() {
        if (xVel != 0) {
            if (xVel < 0) {
                xVel += fric;
            }
            else {
                xVel -= fric;
            }
        }
    }
    public boolean tryMoveNormal() {
        int origX = getX();
        int origY = getY();
        setLocation(origX + xVel, origY + yVel);
        if (!isTouching(PlatformSolid.class)) {
            return true;
        }
        setLocation(origX, origY);
        return false;
    }
    public boolean tryWalk() {
        int origX = getX();
        int origY = getY();
        setLocation(origX + xVel, origY);
        if (!isTouching(PlatformSolid.class)) {
            yVel = 0;
            return true;
        }
        setLocation(origX, origY);
        return false;
    }
    public boolean tryStep() {
        int origX = getX();
        int origY = getY();
        setLocation(origX + xVel, origY - step);
        if (!isTouching(PlatformSolid.class)) {
            yVel = 0;
            return true;
        }
        setLocation(origX, origY);
        return false;
    }
    public boolean trySlide() {
        int origX = getX();
        int origY = getY();
        setLocation(origX, origY + yVel);
        if (!isTouching(PlatformSolid.class)) {
            xVel = 0;
            return true;
        }
        setLocation(origX, origY);
        return false;
    }
    public boolean tryStop() {
        int origX = getX();
        int origY = getY();
        setLocation(origX, origY);
        if (!isTouching(PlatformSolid.class)) {
            xVel = 0;
            yVel = 0;
            return true;
        }
        return false;
    }
    public void escape() {
        PlatformSolid touching; 
        for (int i = ltcy; i > 0; i--) {
            touching = (PlatformSolid) getOneIntersectingObject(PlatformSolid.class);
            if (touching == null) return;
            setLocation(getX() + (getX() > touching.getX() ? 1 : -1), getY() + (getY() > touching.getY() ? 1 : -1));
        }
    }
    public void handleCollisions() {
        if (tryMoveNormal()) return;
        if (tryWalk()) return;
        if (tryStep()) return;
        if (trySlide()) return;
        if (tryStop()) return;
        escape();
    }
    public void handleInput() {
        if (pressingRight()) {
            goRight();
        }
        if (pressingLeft()) {
            goLeft();
        }
        if (pressingJump()) {
            tryJump();
        }
    }
    public void wrapAtEdge() {
        if (isAtEdge()) {
            if (getX() <= 0) {
                super.setLocation(getWorld().getWidth() - 2, getY());
            }
            if (getX() >= getWorld().getWidth() - 1) {
                super.setLocation(1, getY());
            }
            if (getY() <= 0) {
                super.setLocation(getX(), getWorld().getHeight() - 2);
            }
            if (getY() >= getWorld().getHeight() - 1) {
                super.setLocation(getX(), 1);
            }
        }
    }
    public void setLocation(int x, int y) {
        super.setLocation(x, y);
        wrapAtEdge();
    }
    public void work() 
    {
        handleInput();
        applyGravity();
        applyFriction();
        handleCollisions();
    }
    public void act() {
        if (!((PlatformWorld) ((ScrollingWorld.ScrolledWorld) getWorld()).getOverlaying()).isPaused()) {
            work();
        }
    }
}
