import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.util.List;

/**
 * The PlatformWorld is a ScrollingWorld for the PlatformSolids to inhabit. It's like a sidescroller.
 * 
 * @author BenMcBen
 * @version 10/26/2014
 */
public class PlatformWorld extends ScrollingWorld {
    private boolean paused;
    public PlatformWorld() {
        super();
        setupBackground();
        populate();
        paused = true;
    }
    public void setupBackground() {
        GreenfootImage bg = getPersBackground();
        bg.setColor(Color.lightGray);
        bg.fill();
        for (Color color : new Color[]{Color.darkGray, Color.gray, Color.black,
                                       Color.white, Color.red, Color.green,
                                       Color.orange, Color.cyan, Color.yellow}) {
            bg.setColor(color);
            drawStars(bg);
        }
        bg.setColor(Color.white);
        drawClouds(bg);
        drawWater(bg);
    }
    private void drawStars(GreenfootImage img) {
        for (int i = 0; i < 50; i++) {
            img.fillOval(Greenfoot.getRandomNumber(img.getWidth()),
                         Greenfoot.getRandomNumber(img.getHeight()),
                         2, 2);
        }
    }
    private void drawWater(GreenfootImage img) {
        img.setColor(Color.blue);
        img.fillRect(0, 2*img.getHeight()/3, img.getWidth(), img.getHeight()/3);
        img.setColor(Color.gray);
        for (int i = 0; i < 100; i++) {
            img.fillRect(Greenfoot.getRandomNumber(img.getWidth()),
                         2*img.getHeight()/3 + (int) Math.pow(Greenfoot.getRandomNumber((int) Math.sqrt(81*img.getHeight()/3)), 2)/81 + 1,
                         16, 1);
        }
    }
    private void drawClouds(GreenfootImage img) {
        Color initColor = img.getColor();
        img.setColor(new Color(initColor.getRed(), initColor.getGreen(), initColor.getBlue(), 128));
        for (int i = 0; i < 60; i++) {
            img.fillOval(Greenfoot.getRandomNumber(img.getWidth()) - 24,
                         (int) Math.pow(Greenfoot.getRandomNumber((int) Math.sqrt(img.getHeight()/3)), 2),
                         Greenfoot.getRandomNumber(32) + 48,
                         Greenfoot.getRandomNumber(16) + 24);
        }
        img.setColor(initColor);
    }
    public void populate() {
        getUnderlying().addObject(new God(), getUnderlying().getWidth()/2, getUnderlying().getHeight()/2);
        getUnderlying().addObject(new PlatformSolid(), getUnderlying().getWidth()/2, getUnderlying().getHeight()/2 + 100);
        getUnderlying().addObject(new PlatformSolid(), getUnderlying().getWidth()/2 - 50, getUnderlying().getHeight()/2 + 100);
        getUnderlying().addObject(new PlatformSolid(), getUnderlying().getWidth()/2 + 50, getUnderlying().getHeight()/2 + 100);
    }
    public void act() {
        super.act();
        if (Greenfoot.isKeyDown("r")) {
            pause();
            for (God player : (List<God>) getUnderlying().getObjects(God.class)) {
                getUnderlying().removeObject(player);
            }
            getUnderlying().addObject(new God(), getUnderlying().getWidth()/2, getUnderlying().getHeight()/2);
        }
    }
    public boolean isPaused() {
        return paused;
    }
    public void pause() {
        paused = true;
    }
    public void unpause() {
        paused = false;
    }
}
